<?php

class validation {

	private $request; // holds POST or GET array
	private $request_type; // was it post or get request?
	private $errors = array(); // hold errors
	private $names = array(); // array of fieldnames and their english representations

	public function __construct($request_type = 'post'){
		// store superglobal into array to play with
		// could be written better with a variable variable but this works
		if($request_type == 'post'){
			foreach($_POST as $key => $value){
				$this->request[$key] = $value;
			}
			foreach($_FILES as $key => $value){
				$this->request[$key] = $value['name'];
			}
		}elseif($request_type == 'get'){
			foreach($_GET as $key => $value){
				$this->request[$key] = $value;
			}
		}
		$this->request_type = $request_type;
	}
	
	public function __destruct(){
		// puts our array back into the superglobal array
		if($this->request_type == 'post'){
		if(count($_POST) > 0){
			foreach($this->request as $key => $value){
				$_POST[$key] = $value;
			}
		}
		}elseif($this->request_type == 'get'){
			foreach($this->request as $key => $value){
				$_GET[$key] = $value;
			}
		}
	}
	
	// takes the fieldnames and english text names
	// can take array of replacements or will default to convert on the fly
	public function set_fields($input = true){
		$this->names = $input;
	}
	
	// holds errors
	public function errors(){
		foreach($this->errors as $key => $error){
			if(!$error){
				unset($this->errors[$key]);
			}
		}
		return $this->errors;
	}
	
	// parse the validation rules
	public function set_rules($rule_list = array()){
		if(!is_array($rule_list)){
			throw new exception('Rule list must be an array.');
			return false;
		}else{
			// list of rules, fieldname is key, array of specific files is value
			foreach($rule_list as $name => $rules){
				if(strlen($rule_list) < 1){
					throw new exception('Rule list cannot be empty');
					return false;
				}else{
					$rules = explode('|', $rules);
					foreach($rules as $rule){ // each specific rule
						unset($param); // stop carrying over to non-param rules
						if(preg_match("/(.*?)\[(.*?)\]/", $rule, $match)){
							$rule = $match[1];
							$param = $match[2];
						}
						if(method_exists($this, $rule)){ // our methods
							// check if it passes validation and its not already triggered an error
							if($this->$rule($name, $param) !== true && !array_key_exists($name, $this->errors)){
								$this->errors[$name] = $this->$rule($name, $param); // store some kind of generic error string
							}
						}else{
							if(function_exists($rule)){ // any native php function like md5 etc
								$this->request[$name] = $rule($this->request[$name]);  // run variable through native php function
							}else{
								throw new exception('Invalid Rule');
								return false;
							}
						}
					}
				}
			}
		}
	}
	
	// Required field
	private function required($name){
		if(strlen($this->request[$name]) > 0){
			return true;
		}else{
			return $this->replace($name).' is a required field and must be populated.';
		}
	}
	
	// requires form element to only have alphabetical characters
	private function alpha($name){
		if(strlen($this->request[$name]) == 0) return false; // dont check blank values
		if(!preg_match("/^([-a-z ])+$/i", $this->request[$name])){
			return 'The '.$this->replace($name).' field must only be filled with alphabetical characters.';
		}else{
			return true;
		}
	}
		
	// requires form element to only have alphnumeric characters
	private function alnum($name){
		if(strlen($this->request[$name]) == 0) return false; // dont check blank values
		if(!preg_match("/^([-a-z0-9 ])+$/i", $this->request[$name])){
			return 'The '.$this->replace($name).' field must only be filled with alphanumeric characters.';
		}else{
			return true;
		}
	}
		
	// requires form element to only have alphnumeric characters, dashes and underscores
	private function alpha_dash($name){
		if(strlen($this->request[$name]) == 0) return false; // dont check blank values
		if(!preg_match("/^([-a-z0-9_- ])+$/i", $this->request[$name])){
			return 'The '.$this->replace($name).' field must only be filled with alphanumeric characters, dashes and underscores.';
		}else{
			return true;
		}
	}
	
	// requires form element to be a numeric string
	private function num($name){
		if(strlen($this->request[$name]) == 0) return false; // dont check blank values
		if(!is_numeric($this->request[$name])){
			return 'The '.$this->replace($name).' field must only be filled with numeric characters.';
		}else{
			return true;
		}
	}
	
	// requires form element to be valid email
	private function email($name){
		if(strlen($this->request[$name]) == 0) return false; // dont check blank values
		if(!preg_match("/^([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6}$/ix", $this->request[$name])){
			return 'The '.$this->replace($name).' field must be a valid email address.';
		}else{
			return true;
		}
	}
		
	// requires form element to be valid postcode
	private function postcode($name){
		if(strlen($this->request[$name]) == 0) return false; // dont check blank values
		if(!preg_match("/^([A-PR-UWYZa-pr-uwyz0-9][A-HK-Ya-hk-y0-9][ADEGHJMNPRTUVXYadeghjmnprtuvxy0-9]?[ABEHMNPRVWXYabehmnprvwxy0-9]?\s{0,2}[0-9][ABD-HJLN-UW-Zabd-hjln-uw-z]{2})$|^([gG][iI][rR]\s{0,2}0[aA]{2})$/", $this->request[$name])){
			return 'The '.$this->replace($name).' field must be a valid UK postal code.';
		}else{
			return true;
		}
	}
	
	// requires minimum length
	private function min_len($name, $len){
		if(!is_numeric($len)){
			throw new exception('Minimum length must be a number.');
		}
		if(strlen($this->request[$name]) < $len){
			return 'The '.$this->replace($name).' field must at least '.$len.' characters long.';
		}else{
			return true;
		}
	}
	
	// requires mac length
	
	// matches other fieldname
	private function matches($name, $other_name){
		if($this->request[$name] != $this->request[$other_name]){
			return 'The '.$this->replace($name).' field must match the '.$this->replace($other_name).' field.';
		}else{
			return true;
		}	
	}
		
	// Replace fieldname with english name if available
	private function replace($name){
		if(is_array($this->names) && array_key_exists($name, $this->names)){
			return $this->names[$name];
		}else{
			return '&#039;'.ucwords(strtolower(str_replace(array('_', '-'), ' ', $name))).'&#039;';
		}
	}


}







?>